"""Tests related to Galaxy V3 serializers."""

from pulpcore.client.pulp_ansible import (
    ContentCollectionVersionsApi,
    DistributionsAnsibleApi,
    PulpAnsibleApiV3CollectionsApi,
    PulpAnsibleApiV3CollectionsVersionsApi,
    RepositoriesAnsibleApi,
    RemotesCollectionApi,
)
from pulp_smash.pulp3.bindings import monitor_task

from pulp_ansible.tests.functional.utils import SyncHelpersMixin, TestCaseUsingBindings
from pulp_ansible.tests.functional.utils import gen_ansible_client, gen_ansible_remote
from pulp_ansible.tests.functional.utils import set_up_module as setUpModule  # noqa:F401


class CollectionsV3TestCase(TestCaseUsingBindings, SyncHelpersMixin):
    """Test Collections V3 endpoint."""

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.client = gen_ansible_client()
        cls.repo_api = RepositoriesAnsibleApi(cls.client)
        cls.remote_collection_api = RemotesCollectionApi(cls.client)
        cls.distributions_api = DistributionsAnsibleApi(cls.client)
        cls.collections_api = PulpAnsibleApiV3CollectionsApi(cls.client)
        cls.collections_versions_api = ContentCollectionVersionsApi(cls.client)
        cls.collections_versions_v3api = PulpAnsibleApiV3CollectionsVersionsApi(cls.client)

    def test_v3_updated_at(self):
        """Test Collections V3 endpoint field: ``updated_at``."""
        body = gen_ansible_remote(
            url="https://galaxy.ansible.com",
            requirements_file="collections:\n  - pulp.squeezer",
            sync_dependencies=False,
        )
        remote = self.remote_collection_api.create(body)
        self.addCleanup(self.remote_collection_api.delete, remote.pulp_href)

        repo = self._create_repo_and_sync_with_remote(remote)
        distribution = self._create_distribution_from_repo(repo)

        collections = self.collections_api.list(distribution.base_path)

        original_highest_version = collections.data[0].highest_version["version"]
        original_updated_at = collections.data[0].updated_at

        versions = self.collections_versions_api.list(version="0.0.7")
        original_total_versions = self.collections_versions_v3api.list(
            "squeezer", "pulp", distribution.base_path
        ).meta.count

        data = {"remove_content_units": [versions.results[0].pulp_href]}
        response = self.repo_api.modify(repo.pulp_href, data)
        monitor_task(response.task)

        collections = self.collections_api.list(distribution.base_path)
        highest_version = collections.data[0].highest_version["version"]
        updated_at = collections.data[0].updated_at

        total_versions = self.collections_versions_v3api.list(
            "squeezer", "pulp", distribution.base_path
        ).meta.count

        self.assertEqual(highest_version, original_highest_version)
        self.assertEqual(original_total_versions, total_versions + 1)
        self.assertGreater(updated_at, original_updated_at)

    def test_v3_collection_version_from_synced_data(self):
        """Test Collection Versions V3 endpoint fields."""
        body = gen_ansible_remote(
            url="https://galaxy.ansible.com",
            requirements_file="collections:\n  - name: cisco.nxos\n    version: 1.4.0",
            sync_dependencies=False,
        )

        remote = self.remote_collection_api.create(body)
        self.addCleanup(self.remote_collection_api.delete, remote.pulp_href)

        repo = self._create_repo_and_sync_with_remote(remote)
        distribution = self._create_distribution_from_repo(repo)

        version = self.collections_versions_v3api.read(
            "nxos", "cisco", distribution.base_path, "1.4.0"
        )

        self.assertEqual(version.requires_ansible, ">=2.9.10,<2.11")
        self.assertTrue("'name': 'README.md'" in str(version.files))
        self.assertEqual(version.manifest["collection_info"]["name"], "nxos")



import pytest
from pulp_smash.pulp3.utils import (
    get_added_content_summary,
    get_content,
    get_content_summary,
    get_removed_content_summary,
)
from pulp_ansible.tests.functional.utils import (
    gen_ansible_remote,
)
from pulpcore.client.pulp_ansible import AnsibleRepositorySyncURL


def test_add_mark_to_collection_version(
    build_and_upload_collection,
    ansible_repo_api_client,
    ansible_repo,
    ansible_collection_mark_client,
    monitor_task,
):
    """Test adding a mark to a collection version.

    1. Build and upload a collection
    2. Add it to a repo version
    3. Add a mark to the collection version
    4. Assert the mark was added correctly
    5. Remove the mark
    6. Assert the mark was removed correctly
    """
    collection, collection_url = build_and_upload_collection()

    # add it to a repo version
    body = {"add_content_units": [collection_url]}
    monitor_task(ansible_repo_api_client.modify(ansible_repo.pulp_href, body).task)
    ansible_repo = ansible_repo_api_client.read(ansible_repo.pulp_href)

    # assert the content was added correctly
    assert ansible_repo.latest_version_href.endswith("/1/")
    assert get_content_summary(ansible_repo.to_dict()) == {"ansible.collection_version": 1}

    # Add a mark to the collection version
    body = {"content_units": [collection_url], "value": "testable"}
    monitor_task(ansible_repo_api_client.mark(ansible_repo.pulp_href, body).task)
    ansible_repo = ansible_repo_api_client.read(ansible_repo.pulp_href)

    # assert the repo version incremented
    assert ansible_repo.latest_version_href.endswith("/2/")
    # Ensure mark with value "testable" is present on the marks
    marks = ansible_collection_mark_client.list(marked_collection=collection_url).results
    assert len(marks) == 1
    assert marks[0].value == "testable"
    assert marks[0].marked_collection == collection_url

    # Ensure mark is added to the repo
    assert get_added_content_summary(ansible_repo.to_dict()) == {"ansible.collection_mark": 1}

    # Unmark the collection version
    body = {"content_units": [collection_url], "value": "testable"}
    monitor_task(ansible_repo_api_client.unmark(ansible_repo.pulp_href, body).task)

    # Refresh the repo version
    ansible_repo = ansible_repo_api_client.read(ansible_repo.pulp_href)
    # ensure the repo version incremented
    assert ansible_repo.latest_version_href.endswith("/3/")
    # Ensure mark is removed from repo contents
    assert get_removed_content_summary(ansible_repo.to_dict()) == {"ansible.collection_mark": 1}


@pytest.fixture
def distro_serving_one_marked_one_unmarked_collection(
    build_and_upload_collection,
    ansible_repo,
    ansible_repo_api_client,
    ansible_distribution_factory,
    monitor_task,
):
    """Create a distro serving two collections, one marked, one unmarked."""
    collections = []
    for i in range(2):
        _, collection_url = build_and_upload_collection()
        collections.append(collection_url)

    body = {"add_content_units": collections}
    monitor_task(ansible_repo_api_client.modify(ansible_repo.pulp_href, body).task)

    body = {"content_units": collections[:1], "value": "marked-on-sync-test"}
    monitor_task(ansible_repo_api_client.mark(ansible_repo.pulp_href, body).task)

    return ansible_distribution_factory(ansible_repo)


def test_sync_marks(
    ansible_repo_factory,
    distro_serving_one_marked_one_unmarked_collection,
    ansible_remote_collection_api_client,
    gen_object_with_cleanup,
    ansible_repo_api_client,
    monitor_task,
):
    """Test that marks are also synced."""
    distro = distro_serving_one_marked_one_unmarked_collection
    new_repo = ansible_repo_factory()

    # Create Remote
    body = gen_ansible_remote(distro.client_url, include_pulp_auth=True)
    remote = gen_object_with_cleanup(ansible_remote_collection_api_client, body)

    # Sync
    repository_sync_data = AnsibleRepositorySyncURL(remote=remote.pulp_href)
    sync_response = ansible_repo_api_client.sync(new_repo.pulp_href, repository_sync_data)
    monitor_task(sync_response.task)
    repo = ansible_repo_api_client.read(new_repo.pulp_href)

    content_response = get_content(repo.to_dict())
    assert len(content_response["ansible.collection_version"]) == 2
    assert len(content_response["ansible.collection_mark"]) == 1



import re

from django.conf import settings
from django.shortcuts import get_object_or_404, HttpResponse
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import generics, pagination, response, views
from rest_framework.reverse import reverse

from pulpcore.plugin.models import PulpTemporaryFile
from pulpcore.plugin.viewsets import OperationPostponedResponse
from pulpcore.plugin.models import ContentArtifact

from pulp_ansible.app.galaxy.mixins import UploadGalaxyCollectionMixin
from pulp_ansible.app.models import AnsibleDistribution, Collection, CollectionVersion, Role

from .serializers import (
    GalaxyCollectionSerializer,
    GalaxyCollectionUploadSerializer,
    GalaxyCollectionVersionSerializer,
    GalaxyRoleSerializer,
    GalaxyRoleVersionSerializer,
)


class DistributionMixin:
    """
    Mixin for getting the content for the view.
    """

    @property
    def _repository_version(self):
        """Returns repository version."""
        path = self.kwargs["path"]

        context = getattr(self, "pulp_context", None)
        if context and context.get(path, None):
            return self.pulp_context[path]

        distro = get_object_or_404(AnsibleDistribution, base_path=path)
        if distro.repository_version:
            self.pulp_context = {path: distro.repository_version}
            return distro.repository_version

        repo_version = distro.repository.latest_version() if distro.repository else None
        self.pulp_context = {path: repo_version}
        return repo_version

    @property
    def _distro_content(self):
        """Returns distribution content."""
        repo_version = self._repository_version
        if repo_version is None:
            return self.model.objects.none()

        return repo_version.content

    def get_serializer_context(self):
        """Inserts distribution path to a serializer context."""
        context = super().get_serializer_context()
        if "path" in self.kwargs:
            context["path"] = self.kwargs["path"]
        return context


class GalaxyVersionView(views.APIView):
    """
    APIView for Galaxy versions.
    """

    authentication_classes = []
    permission_classes = []

    v3_only = False

    def get(self, request, **kwargs):
        """
        Return a response to the "GET" action.
        """
        available_versions = {"v1": "v1/", "v2": "v2/", "v3": "v3/"}
        if self.v3_only:
            available_versions = {"v3": "v3/"}

        api_info = {
            "available_versions": available_versions,
            "current_version": "v3",
        }

        return response.Response(api_info)


class RoleList(DistributionMixin, generics.ListAPIView):
    """
    APIView for Roles.
    """

    model = Role
    serializer_class = GalaxyRoleSerializer
    authentication_classes = []
    permission_classes = []

    def get_queryset(self):
        """
        Get the list of items for this view.
        """
        roles = Role.objects.distinct("namespace", "name").filter(pk__in=self._distro_content)

        namespace = self.request.query_params.get("owner__username", None)
        if namespace:
            roles = roles.filter(namespace__iexact=namespace)
        name = self.request.query_params.get("name", None)
        if name:
            roles = roles.filter(name__iexact=name)

        return roles


@extend_schema_view(get=extend_schema(operation_id="api_v1_roles_versions_list"))
class RoleVersionList(DistributionMixin, generics.ListAPIView):
    """
    APIView for Role Versions.
    """

    model = Role
    serializer_class = GalaxyRoleVersionSerializer
    authentication_classes = []
    permission_classes = []

    def get_queryset(self):
        """
        Get the list of items for this view.
        """
        namespace, name = re.split(r"\.", self.kwargs["role_pk"])
        versions = Role.objects.filter(pk__in=self._distro_content, name=name, namespace=namespace)
        return versions


class GalaxyCollectionDetailView(DistributionMixin, generics.RetrieveAPIView):
    """
    View for a Collection Detail.
    """

    model = Collection
    serializer_class = GalaxyCollectionSerializer
    authentication_classes = []
    permission_classes = []

    def get(self, request, path=None, namespace=None, name=None):
        """
        Get the detail view of a Collection.
        """
        # This seems wrong, no repository scoping occurring
        collection = get_object_or_404(Collection, namespace=namespace, name=name)
        context = self.get_serializer_context()
        return response.Response(GalaxyCollectionSerializer(collection, context=context).data)


class GalaxyCollectionView(DistributionMixin, UploadGalaxyCollectionMixin, generics.ListAPIView):
    """
    View for Collection models.
    """

    model = Collection
    serializer_class = GalaxyCollectionSerializer
    authentication_classes = []
    permission_classes = []
    pagination_class = pagination.PageNumberPagination

    def get_queryset(self):
        """
        Get the list of Collections for this view.
        """
        return Collection.objects.filter(versions__pk__in=self._distro_content).distinct()

    @extend_schema(deprecated=True)
    def post(self, request, path):
        """
        Queues a task that creates a new Collection from an uploaded artifact.
        """
        distro = get_object_or_404(AnsibleDistribution, base_path=path)
        if not distro.repository and not distro.repository_version:
            return HttpResponse(status=400, reason="Distribution has no repository.")

        serializer = GalaxyCollectionUploadSerializer(
            data=request.data, context={"request": request}
        )
        serializer.is_valid(raise_exception=True)

        temp_file = PulpTemporaryFile.init_and_validate(serializer.validated_data["file"])
        temp_file.save()

        async_result = self._dispatch_import_collection_task(temp_file.pk, distro.repository)
        return OperationPostponedResponse(async_result, request)


@extend_schema_view(get=extend_schema(operation_id="api_v2_collection_versions_list"))
class GalaxyCollectionVersionList(DistributionMixin, generics.ListAPIView):
    """
    APIView for Collections by namespace/name.
    """

    model = CollectionVersion
    serializer_class = GalaxyCollectionVersionSerializer
    pagination_class = pagination.PageNumberPagination
    authentication_classes = []
    permission_classes = []

    def get_queryset(self):
        """
        Get the list of items for this view.
        """
        collection = get_object_or_404(
            Collection, namespace=self.kwargs["namespace"], name=self.kwargs["name"]
        )
        versions = collection.versions.filter(pk__in=self._distro_content)

        return versions


class GalaxyCollectionVersionDetail(DistributionMixin, generics.GenericAPIView):
    """
    APIView for Galaxy Collections Detail view.
    """

    authentication_classes = []
    permission_classes = []

    def get(self, request, path, namespace, name, version):
        """
        Return a response to the "GET" action.
        """
        version = CollectionVersion.objects.get(
            collection__namespace=namespace, collection__name=name, version=version
        )

        get_object_or_404(
            ContentArtifact, content__in=self._distro_content, relative_path=version.relative_path
        )

        # Normally would just pass request to reverse and DRF would automatically build the
        # absolute URI for us. However there's a weird bug where, because there's a kwarg
        # in the URL for this view called "version" (the collection version), DRF gets
        # confused and thinks we're using a versioning scheme
        # (https://www.django-rest-framework.org/api-guide/versioning/) and attempts to insert
        # the version into kwargs, which causes the reverse lookup to fail.
        path = reverse(
            settings.ANSIBLE_URL_NAMESPACE + "collection-artifact-download",
            kwargs={"distro_base_path": self.kwargs["path"], "filename": version.relative_path},
        )
        context = self.get_serializer_context()
        request = context["request"]
        download_url = request.build_absolute_uri(path)

        data = GalaxyCollectionVersionSerializer(version, context=context).data
        data["download_url"] = download_url
        return response.Response(data)



from rest_framework import viewsets

from drf_spectacular.utils import OpenApiParameter, extend_schema, extend_schema_view

from pulp_ansible.app.galaxy.v3.pagination import LimitOffsetPagination

from pulp_ansible.app.galaxy.v3.serializers import (
    CollectionVersionSearchListSerializer,
)
from pulp_ansible.app.galaxy.mixins import GalaxyAuthMixin

from pulpcore.plugin.util import get_url

from pulp_ansible.app.models import CrossRepositoryCollectionVersionIndex, AnsibleDistribution

from pulp_ansible.app.galaxy.v3.filters import CollectionVersionSearchFilter
from pulp_ansible.app.tasks.collectionversion_index import rebuild_index

from pulpcore.plugin.tasking import dispatch
from pulpcore.plugin.viewsets import OperationPostponedResponse


@extend_schema_view(
    list=extend_schema(
        parameters=[
            OpenApiParameter(
                name="repository",
                type={"type": "array", "items": {"type": "string"}},
                location=OpenApiParameter.QUERY,
                required=False,
                style="form",
                explode=True,
                description="Filter collectionversions that are in these repository ids.",
            ),
            OpenApiParameter(
                name="repository_name",
                type={"type": "array", "items": {"type": "string"}},
                location=OpenApiParameter.QUERY,
                required=False,
                style="form",
                explode=True,
                description="Filter collectionversions that are in these repositories.",
            ),
            OpenApiParameter(
                name="distribution",
                type={"type": "array", "items": {"type": "string"}},
                location=OpenApiParameter.QUERY,
                required=False,
                style="form",
                explode=True,
                description="Filter collectionversions that are in these distrubtion ids.",
            ),
            OpenApiParameter(
                name="distribution_base_path",
                type={"type": "array", "items": {"type": "string"}},
                location=OpenApiParameter.QUERY,
                required=False,
                style="form",
                explode=True,
                description="Filter collectionversions that are in these base paths.",
            ),
        ],
    )
)
class CollectionVersionSearchViewSet(GalaxyAuthMixin, viewsets.ModelViewSet):
    """
    A viewset for cross-repo searches.
    """

    serializer_class = CollectionVersionSearchListSerializer
    pagination_class = LimitOffsetPagination
    filterset_class = CollectionVersionSearchFilter
    # This is a dummy, to make the model available to drf-spectacular
    queryset = CrossRepositoryCollectionVersionIndex.objects.none()

    def urlpattern(*args, **kwargs):
        """Helper for galaxy_ng access control."""
        return "pulp_ansible/v3/search/collection_versions"

    def get_queryset(self):
        qs = (
            CrossRepositoryCollectionVersionIndex.objects.select_related("repository")
            .select_related("collection_version")
            .select_related("repository_version")
            .select_related("namespace_metadata")
            .all()
        )

        for permission_class in self.get_permissions():
            if hasattr(permission_class, "scope_queryset"):
                qs = permission_class.scope_queryset(self, qs)

        return qs

    def rebuild(self, request, *args, **kwargs):
        async_result = dispatch(
            rebuild_index,
            exclusive_resources=[
                "/pulp_ansible/galaxy/default/api/v3/plugin/ansible/search/collection-versions/"
            ],
            shared_resources=[get_url(AnsibleDistribution)],
        )
        return OperationPostponedResponse(async_result, request)



"""Tests related to sync ansible plugin collection content type."""

from tempfile import NamedTemporaryFile
from pulp_smash.pulp3.bindings import PulpTestCase, delete_orphans, monitor_task
from pulp_smash.utils import http_get

from pulpcore.client.pulp_ansible import ContentCollectionVersionsApi


from pulp_ansible.tests.functional.constants import (
    ANSIBLE_DEMO_COLLECTION_REQUIREMENTS as DEMO_REQUIREMENTS,
    GALAXY_ANSIBLE_BASE_URL,
)
from pulp_ansible.tests.functional.utils import gen_ansible_client, skip_if
from pulp_ansible.tests.functional.utils import set_up_module as setUpModule  # noqa:F401


def test_tags_filter(
    ansible_collection_version_api_client,
    ansible_collection_remote_factory,
    ansible_sync_factory,
):
    """Filter CollectionVersions by using the tags filter.

    This test targets the following issue:

    * `Pulp #5571 <https://pulp.plan.io/issues/5571>`_

    Do the following:

    1. Create a repository, and a remote.
    2. Sync the remote.
    3. Attempt to filter the CollectionVersions by different tags

    Note that the testing.k8s_demo_collection collection has tags 'k8s' and 'kubernetes'.
    """
    ansible_sync_factory(
        remote=ansible_collection_remote_factory(
            url=GALAXY_ANSIBLE_BASE_URL, requirements_file=DEMO_REQUIREMENTS
        ).pulp_href
    )

    # filter collection versions by tags
    params = {"tags": "nada"}
    collections = ansible_collection_version_api_client.list(**params).results
    assert len(collections) == 0, collections

    params = {"tags": "k8s"}
    collections = ansible_collection_version_api_client.list(**params).results
    assert len(collections) == 1, collections

    params = {"tags": "k8s,kubernetes"}
    collections = ansible_collection_version_api_client.list(**params).results
    assert len(collections) == 1, collections

    params = {"tags": "nada,k8s"}
    collections = ansible_collection_version_api_client.list(**params).results
    assert len(collections) == 0, collections


class ContentUnitTestCase(PulpTestCase):
    """CRUD content unit.

    This test targets the following issues:

    * `Pulp #2872 <https://pulp.plan.io/issues/2872>`_
    * `Pulp #3445 <https://pulp.plan.io/issues/3445>`_
    * `Pulp Smash #870 <https://github.com/pulp/pulp-smash/issues/870>`_
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variable."""
        delete_orphans()
        cls.content_unit = {}
        cls.cv_content_api = ContentCollectionVersionsApi(gen_ansible_client())

    @classmethod
    def tearDownClass(cls):
        """Clean class-wide variable."""
        delete_orphans()

    def upload_collection(self, namespace="pulp", name="squeezer", version="0.0.9"):
        """Upload collection."""
        url = f"https://galaxy.ansible.com/download/{namespace}-{name}-{version}.tar.gz"
        collection_content = http_get(url)
        with NamedTemporaryFile() as temp_file:
            temp_file.write(collection_content)
            return self.cv_content_api.create(
                file=temp_file.name,
                expected_namespace=namespace,
                expected_name=name,
                expected_version=version,
            )

    def test_01_create_content_unit(self):
        """Create content unit."""
        attrs = dict(namespace="pulp", name="squeezer", version="0.0.9")
        response = self.upload_collection(**attrs)
        created_resources = monitor_task(response.task).created_resources
        content_unit = self.cv_content_api.read(created_resources[0])
        self.content_unit.update(content_unit.to_dict())
        for key, val in attrs.items():
            with self.subTest(key=key):
                self.assertEqual(self.content_unit[key], val)

    @skip_if(bool, "content_unit", False)
    def test_02_read_content_unit(self):
        """Read a content unit by its href."""
        content_unit = self.cv_content_api.read(self.content_unit["pulp_href"]).to_dict()
        for key, val in self.content_unit.items():
            with self.subTest(key=key):
                self.assertEqual(content_unit[key], val)

    @skip_if(bool, "content_unit", False)
    def test_02_read_content_units(self):
        """Read a content unit by its pkg_id."""
        page = self.cv_content_api.list(
            namespace=self.content_unit["namespace"], name=self.content_unit["name"]
        )
        self.assertEqual(len(page.results), 1)
        for key, val in self.content_unit.items():
            with self.subTest(key=key):
                self.assertEqual(page.results[0].to_dict()[key], val)

    @skip_if(bool, "content_unit", False)
    def test_03_partially_update(self):
        """Attempt to update a content unit using HTTP PATCH.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        attrs = {"name": "testing"}
        with self.assertRaises(AttributeError) as exc:
            self.cv_content_api.partial_update(self.content_unit["pulp_href"], attrs)
        msg = "object has no attribute 'partial_update'"
        self.assertIn(msg, exc.exception.args[0])

    @skip_if(bool, "content_unit", False)
    def test_03_fully_update(self):
        """Attempt to update a content unit using HTTP PUT.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        attrs = {"name": "testing"}
        with self.assertRaises(AttributeError) as exc:
            self.cv_content_api.update(self.content_unit["pulp_href"], attrs)
        msg = "object has no attribute 'update'"
        self.assertIn(msg, exc.exception.args[0])

    @skip_if(bool, "content_unit", False)
    def test_04_delete(self):
        """Attempt to delete a content unit using HTTP DELETE.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        with self.assertRaises(AttributeError) as exc:
            self.cv_content_api.delete(self.content_unit["pulp_href"])
        msg = "object has no attribute 'delete'"
        self.assertIn(msg, exc.exception.args[0])

    @skip_if(bool, "content_unit", False)
    def test_05_duplicate_no_error(self):
        """Attempt to create duplicate collection."""
        attrs = dict(namespace="pulp", name="squeezer", version="0.0.9")
        response = self.upload_collection(**attrs)
        created_resources = monitor_task(response.task).created_resources
        self.assertEqual(created_resources[0], self.content_unit["pulp_href"])



"""Tests collection sync functionality that is common to both Galaxy V2 and V3."""

import pytest
import subprocess


@pytest.fixture
def sync_and_count(
    ansible_repo_factory,
    ansible_git_remote_factory,
    ansible_sync_factory,
    ansible_collection_version_api_client,
):
    """A helper fixture to perform a sync and return the count in the repo now."""

    def _sync_and_count(remote_body, repo=None, **sync_kwargs):
        if repo is None:
            repo = ansible_repo_factory()
        previous_latest = repo.latest_version_href
        remote = ansible_git_remote_factory(**remote_body)
        sync_kwargs.update({"remote": remote.pulp_href})
        repo = ansible_sync_factory(repo, **sync_kwargs)
        assert repo.latest_version_href != previous_latest
        c = ansible_collection_version_api_client.list(repository_version=repo.latest_version_href)
        return c.count

    yield _sync_and_count


@pytest.mark.parallel
def test_sync_collection_from_git(
    sync_and_count,
    ansible_repo_factory,
    ansible_distribution_factory,
    gen_user,
    gen_object_with_cleanup,
    ansible_dir_factory,
):
    """Sync collections from Git repositories and then install one of them."""
    body = {
        "url": "https://github.com/pulp/pulp_installer.git",
        "include_pulp_auth": True,
    }

    repo = ansible_repo_factory()
    count = sync_and_count(body, repo=repo)
    assert count == 1

    count = sync_and_count(body, repo=repo)
    assert count == 1

    body = {"url": "https://github.com/pulp/squeezer.git"}
    count = sync_and_count(body, repo=repo)
    assert count == 2

    # Create a distribution.
    distribution = ansible_distribution_factory(repo)

    collection_name = "pulp.squeezer"

    ansible_dir = ansible_dir_factory(distribution.client_url)

    directory = ansible_dir / "ansible_collections" / collection_name.replace(".", "/")

    assert not directory.exists(), f"Directory {directory} already exists"

    # The install command needs --pre so a pre-release collection versions install
    subprocess.run(
        (
            "ansible-galaxy",
            "collection",
            "install",
            "--pre",
            collection_name,
            "-c",
            "-p",
            ansible_dir,
        ),
        cwd=ansible_dir,
        check=True,
    )

    assert directory.exists(), f"Could not find directory {directory}"


@pytest.mark.parallel
def test_sync_metadata_only_collection_from_git(
    ansible_repo,
    sync_and_count,
    ansible_distribution_factory,
    galaxy_v3_collection_versions_api_client,
):
    """Sync collections from Git repositories with metadata_only=True."""
    body = {
        "url": "https://github.com/ansible-collections/amazon.aws/",
        "metadata_only": True,
        "git_ref": "2.1.0",
    }
    count = sync_and_count(body, repo=ansible_repo)
    assert count == 1

    # Create a distribution.
    distribution = ansible_distribution_factory(ansible_repo)

    galaxy_v3_collection_versions_api_client.read("aws", "amazon", distribution.base_path, "2.1.0")


@pytest.mark.parallel
def test_sync_collection_from_git_commit_sha(
    ansible_repo,
    sync_and_count,
    ansible_distribution_factory,
    galaxy_v3_collection_versions_api_client,
):
    """Sync collections from Git repositories with git_ref that is a commit sha."""
    body = {
        "url": "https://github.com/ansible-collections/amazon.aws/",
        "metadata_only": True,
        "git_ref": "d0b54fc082cb63f63d34246c8fe668e19e74777c",
    }

    count = sync_and_count(body, repo=ansible_repo)
    assert count == 1

    # Create a distribution.
    distribution = ansible_distribution_factory(ansible_repo)

    version = galaxy_v3_collection_versions_api_client.read(
        "aws", "amazon", distribution.base_path, "1.5.1"
    )

    assert version.git_url == "https://github.com/ansible-collections/amazon.aws/"
    assert version.git_commit_sha == "d0b54fc082cb63f63d34246c8fe668e19e74777c"
    assert version.download_url is None


@pytest.mark.parallel
def test_sync_metadata_only_collection_from_pulp(
    ansible_sync_factory,
    ansible_repo_factory,
    ansible_distribution_factory,
    ansible_collection_version_api_client,
    ansible_git_remote_factory,
    ansible_collection_remote_factory,
):
    """Sync collections from another Pulp that only has metadata.

    Create a GitRemote
    Create a repository
    Sync from the remote
    Create distribution to serve the repository
    Create CollectionRemote pointing to the distribution.
    Create a second repository.
    Sync using the CollectionRemote into the second repository.
    Assert content is added to the second repository.

    """
    amazon_remote = ansible_git_remote_factory(
        url="https://github.com/ansible-collections/amazon.aws/",
        metadata_only=True,
        git_ref="2.1.0",
        include_pulp_auth=True,
    )

    first_repo = ansible_repo_factory()
    first_repo = ansible_sync_factory(first_repo, remote=amazon_remote.pulp_href)

    content = ansible_collection_version_api_client.list(
        repository_version=first_repo.latest_version_href
    )
    assert content.count == 1

    # Create a distribution.
    distribution = ansible_distribution_factory(first_repo)

    requirements_file = "collections:\n  - amazon.aws"
    second_remote = ansible_collection_remote_factory(
        url=distribution.client_url,
        requirements_file=requirements_file,
        sync_dependencies=False,
        include_pulp_auth=True,
    )

    second_repo = ansible_repo_factory(remote=second_remote.pulp_href)
    second_repo = ansible_sync_factory(second_repo)

    first_content = ansible_collection_version_api_client.list(
        repository_version=f"{first_repo.pulp_href}versions/1/"
    )
    assert first_content.count >= 1
    second_content = ansible_collection_version_api_client.list(
        repository_version=f"{second_repo.pulp_href}versions/1/"
    )
    assert second_content.count >= 1



import pytest
from pulpcore.client.pulp_ansible import (
    # Old APIs
    PulpAnsibleApiV3CollectionsApi,
    PulpAnsibleApiV3CollectionsVersionsApi,
    PulpAnsibleApiV3CollectionVersionsAllApi,
    PulpAnsibleApiV3CollectionsAllApi,
    PulpAnsibleApiV3CollectionsVersionsDocsBlobApi,
    # New APIs
    PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexApi,
    PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexVersionsApi,
    PulpAnsibleApiV3PluginAnsibleContentCollectionsAllVersionsApi,
    PulpAnsibleApiV3PluginAnsibleContentCollectionsAllCollectionsApi,
    PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexVersionsDocsBlobApi,
)


@pytest.fixture(scope="class")
def seeded_distribution(
    ansible_collection_remote_factory,
    ansible_sync_factory,
    ansible_distribution_factory,
    monitor_task,
):
    """Set up the content guard tests."""
    requirements_file = (
        "collections:\n" "  - name: testing.k8s_demo_collection\n" "  - name: pulp.squeezer"
    )
    remote = ansible_collection_remote_factory(
        url="https://galaxy.ansible.com",
        requirements_file=requirements_file,
        sync_dependencies=False,
    )

    repository = ansible_sync_factory(remote=remote.pulp_href)
    distribution = ansible_distribution_factory(repository=repository)
    return distribution


@pytest.fixture(scope="class")
def verify_list(ansible_bindings_client, seeded_distribution):
    def _verify_list(new_api, old_api, query_params, args=[]):
        def is_list_identical(r1, r2):
            d1 = r1.to_dict()
            d2 = r2.to_dict()

            # skip comparing links so that the tests pass when galaxy_ng is installed
            assert d1["meta"] == d2["meta"]
            assert d1["data"] == d2["data"]

        c1 = new_api(ansible_bindings_client)
        c2 = old_api(ansible_bindings_client)

        args_new = [seeded_distribution.base_path, *args]

        # test pagination
        r1 = c1.list(*args_new, limit=1)
        r2 = c2.list(*args, limit=1)

        assert len(r1.data) == 1
        assert len(r2.data) == 1

        is_list_identical(r1, r2)

        # test query params
        r1 = c1.list(*args_new, **query_params)
        r2 = c2.list(*args, **query_params)

        is_list_identical(r1, r2)

        # test no query params
        r1 = c1.list(*args_new)
        r2 = c2.list(*args)

        is_list_identical(r1, r2)

    return _verify_list


@pytest.fixture(scope="class")
def verify_all_collections(ansible_bindings_client, seeded_distribution):
    def _verify_all_collections(new_api, old_api):
        c1 = new_api(ansible_bindings_client)
        c2 = old_api(ansible_bindings_client)
        r1 = c1.list(seeded_distribution.base_path, seeded_distribution.base_path)
        r2 = c2.list(seeded_distribution.base_path)

        assert r1 == r2

    return _verify_all_collections


@pytest.fixture(scope="class")
def verify_read(ansible_bindings_client, seeded_distribution):
    def _verify_read(new_api, old_api, version=None):
        c1 = new_api(ansible_bindings_client)
        c2 = old_api(ansible_bindings_client)

        args = ["k8s_demo_collection", "testing", seeded_distribution.base_path]
        if version:
            args.append(version)

        r1 = c1.read(seeded_distribution.base_path, *args)
        r2 = c2.read(*args)

        assert r1 == r2

    return _verify_read


def test_redirects_are_identical(
    seeded_distribution, verify_list, verify_all_collections, verify_read
):
    """
    Verify new and legacy endpoints are identical.
    """
    verify_list(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexApi,
        PulpAnsibleApiV3CollectionsApi,
        {"namespace": "testing"},
        args=[seeded_distribution.base_path],
    )

    verify_list(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexVersionsApi,
        PulpAnsibleApiV3CollectionsVersionsApi,
        {"is_highest": True},
        args=["k8s_demo_collection", "testing", seeded_distribution.base_path],
    )

    verify_read(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexApi, PulpAnsibleApiV3CollectionsApi
    )

    verify_read(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexVersionsApi,
        PulpAnsibleApiV3CollectionsVersionsApi,
        version="0.0.3",
    )

    verify_read(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsIndexVersionsDocsBlobApi,
        PulpAnsibleApiV3CollectionsVersionsDocsBlobApi,
        version="0.0.3",
    )

    verify_all_collections(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsAllVersionsApi,
        PulpAnsibleApiV3CollectionVersionsAllApi,
    )

    verify_all_collections(
        PulpAnsibleApiV3PluginAnsibleContentCollectionsAllCollectionsApi,
        PulpAnsibleApiV3CollectionsAllApi,
    )



#!/usr/bin/env python3

from setuptools import setup, find_packages

with open("requirements.txt") as requirements:
    requirements = requirements.readlines()

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="pulp-ansible",
    version="0.20.8",
    description="Pulp plugin to manage Ansible content, e.g. roles",
    long_description=long_description,
    license="GPLv2+",
    author="Pulp Ansible Plugin Project Developers",
    author_email="pulp-dev@redhat.com",
    url="https://github.com/pulp/pulp_ansible",
    install_requires=requirements,
    extras_require={
        "test": [
            "coverage",
            "dynaconf",
            "flake8",
            "flake8-docstrings",
            "flake8-quotes",
            "flake8-tuple",
            "mock",
            "orionutils",
            "pulp-smash",
            "pytest",
        ]
    },
    include_package_data=True,
    packages=find_packages(exclude=["tests"]),
    classifiers=(
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: POSIX :: Linux",
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ),
    entry_points={
        "pulpcore.plugin": ["pulp_ansible = pulp_ansible:default_app_config"],
        "pytest11": ["pulp_ansible = pulp_ansible.tests.functional"],
    },
)



"""
Tests PulpExporter and PulpExport functionality.

NOTE: assumes ALLOWED_EXPORT_PATHS setting contains "/tmp" - all tests will fail if this is not
the case.
"""

import pytest
import uuid

from pulpcore.client.pulp_ansible import AnsibleRepositorySyncURL
from pulp_ansible.tests.functional.constants import ANSIBLE_FIXTURE_URL


@pytest.mark.parallel
def test_export_then_import(
    gen_object_with_cleanup,
    ansible_repo_factory,
    ansible_collection_remote_factory,
    ansible_role_remote_factory,
    ansible_repo_api_client,
    ansible_repo_version_api_client,
    ansible_distribution_factory,
    galaxy_v3_plugin_namespaces_api_client,
    exporters_pulp_api_client,
    exporters_pulp_exports_api_client,
    importers_pulp_api_client,
    importers_pulp_imports_api_client,
    ascii_armored_detached_signing_service,
    random_image_factory,
    monitor_task,
    monitor_task_group,
):
    """Issue and evaluate a PulpExport (tests both Create and Read)."""
    # Prepare content
    remote_a = ansible_collection_remote_factory(
        url="https://galaxy.ansible.com",
        requirements_file="collections:\n  - testing.k8s_demo_collection",
    )
    remote_b = ansible_role_remote_factory(url=ANSIBLE_FIXTURE_URL)
    repo_a = ansible_repo_factory()
    repo_b = ansible_repo_factory()
    sync_response_a = ansible_repo_api_client.sync(
        repo_a.pulp_href, AnsibleRepositorySyncURL(remote=remote_a.pulp_href)
    )
    sync_response_b = ansible_repo_api_client.sync(
        repo_b.pulp_href, AnsibleRepositorySyncURL(remote=remote_b.pulp_href)
    )
    monitor_task(sync_response_a.task)
    monitor_task(sync_response_b.task)
    signing_body = {
        "signing_service": ascii_armored_detached_signing_service.pulp_href,
        "content_units": ["*"],
    }
    monitor_task(ansible_repo_api_client.sign(repo_a.pulp_href, signing_body).task)
    repo_ver_a = ansible_repo_version_api_client.read(f"{repo_a.pulp_href}versions/2/")
    repo_ver_b = ansible_repo_version_api_client.read(f"{repo_b.pulp_href}versions/1/")

    mark_body = {
        "content_units": ["*"],
        "value": "exportable-mark",
    }
    monitor_task(ansible_repo_api_client.mark(repo_a.pulp_href, mark_body).task)
    repo_ver_a = ansible_repo_version_api_client.read(f"{repo_a.pulp_href}versions/3/")

    distro = ansible_distribution_factory(repository=repo_a)
    namespace_body = {
        "name": "example",
        "avatar": random_image_factory(),
        "links": [
            {"name": "homepage", "url": "https://example.com"},
            {"name": "github", "url": "https://gh.com"},
        ],
    }
    task = galaxy_v3_plugin_namespaces_api_client.create(
        path=distro.base_path, distro_base_path=distro.base_path, **namespace_body
    )
    monitor_task(task.task)
    repo_ver_a = ansible_repo_version_api_client.read(f"{repo_a.pulp_href}versions/4/")

    # Prepare export
    exporter = gen_object_with_cleanup(
        exporters_pulp_api_client,
        {
            "name": str(uuid.uuid4()),
            "path": f"/tmp/{uuid.uuid4()}/",
            "repositories": [repo.pulp_href for repo in [repo_a, repo_b]],
        },
    )

    # Export
    task = exporters_pulp_exports_api_client.create(exporter.pulp_href, {}).task
    task = monitor_task(task)
    assert len(task.created_resources) == 1
    export = exporters_pulp_exports_api_client.read(task.created_resources[0])
    assert export is not None
    assert len(exporter.repositories) == len(export.exported_resources)
    assert export.output_file_info is not None
    for an_export_filename in export.output_file_info.keys():
        assert "//" not in an_export_filename
    export_filename = next(
        f for f in export.output_file_info.keys() if f.endswith("tar.gz") or f.endswith("tar")
    )

    # Prepare import
    repo_c = ansible_repo_factory()
    repo_d = ansible_repo_factory()
    repo_mapping = {repo_a.name: repo_c.name, repo_b.name: repo_d.name}
    importer = gen_object_with_cleanup(
        importers_pulp_api_client, {"name": str(uuid.uuid4()), "repo_mapping": repo_mapping}
    )

    # Import
    import_response = importers_pulp_imports_api_client.create(
        importer.pulp_href, {"path": export_filename}
    )
    monitor_task_group(import_response.task_group)
    repo_c = ansible_repo_api_client.read(repo_c.pulp_href)
    repo_d = ansible_repo_api_client.read(repo_d.pulp_href)
    assert repo_c.latest_version_href == f"{repo_c.pulp_href}versions/1/"
    assert repo_d.latest_version_href == f"{repo_d.pulp_href}versions/1/"
    repo_ver_c = ansible_repo_version_api_client.read(f"{repo_c.pulp_href}versions/1/")
    repo_ver_d = ansible_repo_version_api_client.read(f"{repo_d.pulp_href}versions/1/")
    assert (
        repo_ver_c.content_summary.added["ansible.collection_version"]["count"]
        == repo_ver_a.content_summary.present["ansible.collection_version"]["count"]
    )
    assert (
        repo_ver_c.content_summary.added["ansible.collection_deprecation"]["count"]
        == repo_ver_a.content_summary.present["ansible.collection_deprecation"]["count"]
    )
    assert (
        repo_ver_c.content_summary.added["ansible.collection_signature"]["count"]
        == repo_ver_a.content_summary.present["ansible.collection_signature"]["count"]
    )
    assert (
        repo_ver_c.content_summary.added["ansible.collection_mark"]["count"]
        == repo_ver_a.content_summary.present["ansible.collection_mark"]["count"]
    )
    assert (
        repo_ver_c.content_summary.added["ansible.namespace"]["count"]
        == repo_ver_a.content_summary.present["ansible.namespace"]["count"]
    )
    assert (
        repo_ver_d.content_summary.added["ansible.role"]["count"]
        == repo_ver_b.content_summary.present["ansible.role"]["count"]
    )

    # Import a second time
    import_response = importers_pulp_imports_api_client.create(
        importer.pulp_href, {"path": export_filename}
    )
    monitor_task_group(import_response.task_group)
    assert len(importers_pulp_imports_api_client.list(importer.pulp_href).results) == 2
    for repo in [repo_c, repo_d]:
        repo = ansible_repo_api_client.read(repo.pulp_href)
        # still only one version as pulp won't create a new version if nothing changed
        assert repo.latest_version_href == f"{repo.pulp_href}versions/1/"



import unittest

from pulp_ansible.tests.functional.utils import (
    SyncHelpersMixin,
    TestCaseUsingBindings,
)

from pulpcore.client.pulp_ansible.exceptions import ApiException
from pulp_smash.pulp3.bindings import monitor_task
from orionutils.generator import build_collection, randstr


class CollectionDeletionTestCase(TestCaseUsingBindings, SyncHelpersMixin):
    """Test collection deletion."""

    def upload_to_distro(self, collection_config=None, distro=None):
        """Helper method to upload a collection to the distribution."""
        distro = distro or self.distribution
        repo = distro.repository

        collection_config = collection_config or {}
        config = {"namespace": randstr(), "name": randstr(), "version": "1.0.0"}
        config.update(collection_config)

        col = build_collection("skeleton", config=config)
        response = self.cv_api.create(file=col.filename, repository=repo)
        monitor_task(response.task)
        return {"namespace": col.namespace, "name": col.name, "version": col.version}

    def setUp(self):
        """Set up the collection deletion tests."""
        (self.repo, self.distribution) = self._create_empty_repo_and_distribution()

        self.collection_versions = ["1.0.0", "1.0.1"]
        self.collection_name = randstr()
        self.collection_namespace = randstr()
        config = {"namespace": self.collection_namespace, "name": self.collection_name}

        for version in self.collection_versions:
            config["version"] = version
            self.upload_to_distro(collection_config=config)

    def test_collection_deletion(self):
        """Test deleting an entire collection."""
        collections = self.collections_v3api.list(self.distribution.base_path)
        assert collections.meta.count == 1

        resp = self.collections_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )
        task = monitor_task(resp.task)
        assert len(task.created_resources) == 1

        collections = self.collections_v3api.list(self.distribution.base_path)
        assert collections.meta.count == 0

        versions = self.collections_versions_v3api.list(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )

        assert versions.meta.count == 0

        with self.assertRaises(ApiException) as e:
            self.collections_v3api.read(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
            )

        assert e.exception.status == 404

    def test_collection_version_deletion(self):
        """Test deleting a specific collection version."""
        collections = self.collections_v3api.list(self.distribution.base_path)
        assert collections.meta.count == 1

        versions = self.collections_versions_v3api.list(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )
        assert versions.meta.count == len(self.collection_versions)

        # Delete one version
        to_delete = self.collection_versions.pop()

        resp = self.collections_versions_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
            version=to_delete,
        )

        monitor_task(resp.task)

        with self.assertRaises(ApiException) as e:
            self.collections_versions_v3api.read(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
                version=to_delete,
            )
            assert e.status == 404

        # Verify that the collection still exists
        collections = self.collections_v3api.list(self.distribution.base_path)
        assert collections.meta.count == 1

        # Verify that the other versions still exist
        versions = self.collections_versions_v3api.list(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )
        assert versions.meta.count == len(self.collection_versions)

        # Delete the rest of the versions

        for to_delete in self.collection_versions:
            resp = self.collections_versions_v3api.delete(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
                version=to_delete,
            )

            monitor_task(resp.task)

        # Verify all the versions have been deleted
        versions = self.collections_versions_v3api.list(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )

        assert versions.meta.count == 0

        # With all the versions deleted, verify that the collection has also
        # been deleted
        with self.assertRaises(ApiException) as e:
            self.collections_v3api.read(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
            )
        assert e.exception.status == 404

    def test_invalid_deletion(self):
        """Test deleting collections that are dependencies for other collections."""
        dependent_version = self.collection_versions.pop()
        dependent_collection = self.upload_to_distro(
            collection_config={
                "dependencies": {
                    f"{self.collection_namespace}.{self.collection_name}": dependent_version
                }
            },
        )

        err_msg = f"{dependent_collection['namespace']}.{dependent_collection['name']} 1.0.0"

        # Verify entire collection can't be deleted
        with self.assertRaises(ApiException) as e:
            self.collections_v3api.delete(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
            )

        # check error message includes collection that's blocking delete
        assert err_msg in e.exception.body

        # Verify specific version that's used can't be deleted
        with self.assertRaises(ApiException) as e:
            self.collections_versions_v3api.delete(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
                version=dependent_version,
            )
        assert e.exception.status == 400

        # check error message includes collection that's blocking delete
        assert err_msg in e.exception.body

        # Verify non dependent version can be deleted.
        resp = self.collections_versions_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
            version=self.collection_versions[0],
        )

        resp = monitor_task(resp.task)

    def test_delete_deprecated_content(self):
        """Test that deprecated content is removed correctly."""
        # Deprecate the collection
        result = self.collections_v3api.update(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
            patched_collection={"deprecated": True},
        )
        monitor_task(result.task)

        resp = self.collections_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )
        monitor_task(resp.task)

        # Verify that all the content is gone
        repo = self.repo_api.read(self.repo.pulp_href)
        latest_version = self.repo_version_api.read(repo.latest_version_href)

        assert len(latest_version.content_summary.present) == 0

    @unittest.skip("needs to use signing fixtures from pulpcore")
    def test_delete_signed_content(self):
        """Test that signature content is removed correctly."""
        create_signing_service = None  # Avoids flake8 complaining since this doesn't exist

        sign_service = create_signing_service()

        # Sign the collections
        body = {
            "content_units": [
                "*",
            ],
            "signing_service": sign_service.pulp_href,
        }
        monitor_task(self.repo_api.sign(self.repo.pulp_href, body).task)

        resp = self.collections_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
        )
        monitor_task(resp.task)

        # Verify that all the content is gone
        repo = self.repo_api.read(self.repo.pulp_href)
        latest_version = self.repo_version_api.read(repo.latest_version_href)

        assert len(latest_version.content_summary.present) == 0

    def test_version_deletion_with_range_of_versions(self):
        """Verify collections can be deleted when another version satisfies requirements."""
        # Create a collection that depends on any version of an existing collection
        self.upload_to_distro(
            collection_config={
                "dependencies": {f"{self.collection_namespace}.{self.collection_name}": "*"}
            },
        )

        to_delete = self.collection_versions.pop()

        # Verify the collection version can be deleted as long as there is one version
        # left that satisfies the requirements.
        resp = self.collections_versions_v3api.delete(
            path=self.distribution.base_path,
            name=self.collection_name,
            namespace=self.collection_namespace,
            version=to_delete,
        )

        resp = monitor_task(resp.task)

        # Verify that the last version of the collection can't be deleted

        with self.assertRaises(ApiException) as e:
            self.collections_versions_v3api.delete(
                path=self.distribution.base_path,
                name=self.collection_name,
                namespace=self.collection_namespace,
                version=self.collection_versions[0],
            )

        assert e.exception.status == 400



from pulpcore.plugin import PulpPluginAppConfig


class PulpAnsiblePluginAppConfig(PulpPluginAppConfig):
    """
    Entry point for pulp_ansible plugin.
    """

    name = "pulp_ansible.app"
    label = "ansible"
    version = "0.20.8"
    python_package_name = "pulp-ansible"



def v3_can_view_repo_content(request, view, action):
    """
    Check if the repo is private, only let users with view repository permissions
    view the collections here.
    """

    if "distro_base_path" in view.kwargs:
        repo_version = view._repository_version
        repo = view._repository
        if not repo or not repo_version:
            return False

        repo = repo.cast()

        if repo.private:
            perm = "ansible.view_ansiblerepository"
            return request.user.has_perm(perm) or request.user.has_perm(perm, repo)

    return True


def v3_can_modify_repo_content(request, view, action):
    """
    Check if the user has Permission to modify Repository content.

    Returns True if the user has model or object-level permissions on the repository.
    False otherwise.
    """

    permission = "ansible.modify_ansible_repo_content"

    if request.user.has_perm(permission):
        return True

    if "distro_base_path" in view.kwargs:
        repo_version = view._repository_version
        repo = view._repository
        if not repo or not repo_version:
            return False

        repo = repo.cast()
        return request.user.has_perm(permission, repo)

    return False



# Generated by Django 2.2.16 on 2020-11-20 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0026_deprecation_per_repository'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='name',
            field=models.CharField(editable=False, max_length=64, unique=True),
        ),
    ]



"""Tests that Collections hosted by Pulp can be installed by ansible-galaxy."""

import json
from os import path
import subprocess
import pytest

from pulp_ansible.tests.functional.constants import (
    ANSIBLE_DEMO_COLLECTION,
    ANSIBLE_DEMO_COLLECTION_VERSION,
    ANSIBLE_DEMO_COLLECTION_REQUIREMENTS as DEMO_REQUIREMENTS,
    GALAXY_ANSIBLE_BASE_URL,
)


@pytest.fixture
def install_scenario_distribution(
    ansible_sync_factory,
    ansible_collection_remote_factory,
    ansible_distribution_factory,
):
    """Prepare a distribution to install from."""
    remote = ansible_collection_remote_factory(
        url=GALAXY_ANSIBLE_BASE_URL, requirements_file=DEMO_REQUIREMENTS
    )
    repo = ansible_sync_factory(remote=remote.pulp_href)
    return ansible_distribution_factory(repo)


# Do not mark this test parallel, because we need to assert that the last dl log is ours.
def test_install_collection(install_scenario_distribution, ansible_dir_factory, pulp_settings):
    """Test that the collection can be installed from Pulp."""
    if not pulp_settings.ANSIBLE_COLLECT_DOWNLOAD_LOG:
        pytest.skip("ANSIBLE_COLLECT_DOWNLOAD_LOG not enabled")

    collection_name = ANSIBLE_DEMO_COLLECTION
    collection_version = ANSIBLE_DEMO_COLLECTION_VERSION

    temp_dir = str(ansible_dir_factory(install_scenario_distribution.client_url))

    directory = "{}/ansible_collections/{}".format(temp_dir, collection_name.replace(".", "/"))

    assert not path.exists(directory), "Directory {} already exists".format(directory)
    subprocess.run(
        (
            "ansible-galaxy",
            "collection",
            "install",
            collection_name,
            "-c",
            "-p",
            temp_dir,
        ),
        cwd=temp_dir,
        check=True,
    )

    assert path.exists(directory), "Could not find directory {}".format(directory)
    dl_log_dump = subprocess.check_output(["pulpcore-manager", "download-log"])
    dl_log = json.loads(dl_log_dump)
    assert (
        dl_log[-1]["content_unit"] == f"<CollectionVersion: {collection_name} {collection_version}>"
    )
    assert dl_log[-1]["user"] == "admin"


@pytest.mark.parallel
def test_install_signed_collection(
    ansible_repo_api_client,
    install_scenario_distribution,
    signing_gpg_homedir_path,
    ascii_armored_detached_signing_service,
    gen_user,
    ansible_dir_factory,
    monitor_task,
):
    """Test that the collection can be installed from Pulp."""
    user = gen_user(model_roles=["ansible.ansiblerepository_owner"])
    with user:
        collection_name = ANSIBLE_DEMO_COLLECTION
        repository_href = install_scenario_distribution.repository
        signing_service = ascii_armored_detached_signing_service
        # Switch this over to signature upload in the future
        signing_body = {"signing_service": signing_service.pulp_href, "content_units": ["*"]}
        monitor_task(ansible_repo_api_client.sign(repository_href, signing_body).task)

        ansible_dir = ansible_dir_factory(install_scenario_distribution.client_url)

        directory = ansible_dir / "ansible_collections" / collection_name.replace(".", "/")

        assert not path.exists(directory), "Directory {} already exists".format(directory)
        subprocess.run(
            (
                "ansible-galaxy",
                "collection",
                "install",
                collection_name,
                "-c",
                "-p",
                ansible_dir,
                "--keyring",
                f"{signing_gpg_homedir_path}/pubring.kbx",
            ),
            cwd=ansible_dir,
            check=True,
        )
        assert path.exists(directory), "Could not find directory {}".format(directory)



# Generated by Django 3.2.5 on 2021-07-23 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0033_swap_distribution_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collectionimport',
            name='messages',
            field=models.JSONField(default=list, editable=False),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='contents',
            field=models.JSONField(default=list, editable=False),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='dependencies',
            field=models.JSONField(default=dict, editable=False),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='docs_blob',
            field=models.JSONField(default=dict, editable=False),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='files',
            field=models.JSONField(default=dict, editable=False),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='manifest',
            field=models.JSONField(default=dict, editable=False),
        ),
    ]
